/*=========================================================================

Program:   VMTK
Module:    $RCSfile: vtkbvsLaplacianSegmentationLevelSetImageFilter.cxx,v $
Language:  C++
Date:      $Date: 2006/04/06 16:48:25 $
Version:   $Revision: 1.1 $

  Copyright (c) Luca Antiga, David Steinman. All rights reserved.
  See LICENCE file for details.

  Portions of this code are covered under the VTK copyright.
  See VTKCopyright.txt or http://www.kitware.com/VTKCopyright.htm 
  for details.

  Portions of this code are covered under the ITK copyright.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm
  for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#include "vtkvmtkLaplacianSegmentationLevelSetImageFilter.h"
#include "vtkObjectFactory.h"

vtkCxxRevisionMacro(vtkvmtkLaplacianSegmentationLevelSetImageFilter, "$Revision: 1.4 $");
vtkStandardNewMacro(vtkvmtkLaplacianSegmentationLevelSetImageFilter);



