#!/usr/bin/env tclsh

# Execute this script each time you add a new directory or file.

# Packages

#pkg_mkIndex -direct -verbose vtkvmtk
#pkg_mkIndex -direct -verbose vtkvmtkcommon
#pkg_mkIndex -direct -verbose vtkvmtkcomputationalgeometry
#pkg_mkIndex -direct -verbose vtkvmtkdifferentialgeometry
#pkg_mkIndex -direct -verbose vtkvmtkio
#pkg_mkIndex -direct -verbose vtkvmtkmisc
#pkg_mkIndex -direct -verbose vtkvmtksegmentation

exit
