/*=========================================================================

  Program:   VMTK
  Module:    $RCSfile: vtkvmtkPolyDataFELaplaceBeltramiStencil.h,v $
  Language:  C++
  Date:      $Date: 2006/04/06 16:46:44 $
  Version:   $Revision: 1.4 $

  Copyright (c) Luca Antiga, David Steinman. All rights reserved.
  See LICENCE file for details.

  Portions of this code are covered under the VTK copyright.
  See VTKCopyright.txt or http://www.kitware.com/VTKCopyright.htm 
  for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
// .NAME vtkvmtkPolyDataFELaplaceBeltramiStencil - ..
// .SECTION Description
// ..

#ifndef __vtkvmtkPolyDataFELaplaceBeltramiStencil_h
#define __vtkvmtkPolyDataFELaplaceBeltramiStencil_h

#include "vtkObject.h"
#include "vtkvmtkConstants.h"
#include "vtkvmtkPolyDataLaplaceBeltramiStencil.h"
#include "vtkvmtkWin32Header.h"

class VTK_VMTK_DIFFERENTIAL_GEOMETRY_EXPORT vtkvmtkPolyDataFELaplaceBeltramiStencil : public vtkvmtkPolyDataLaplaceBeltramiStencil
{
public:

  static vtkvmtkPolyDataFELaplaceBeltramiStencil *New();
  vtkTypeRevisionMacro(vtkvmtkPolyDataFELaplaceBeltramiStencil,vtkvmtkPolyDataLaplaceBeltramiStencil);

  virtual vtkIdType GetItemType() {return VTK_VMTK_FE_LAPLACE_BELTRAMI_STENCIL;};

protected:
  vtkvmtkPolyDataFELaplaceBeltramiStencil();
  ~vtkvmtkPolyDataFELaplaceBeltramiStencil() {};

  void ScaleWithArea();

private:
  vtkvmtkPolyDataFELaplaceBeltramiStencil(const vtkvmtkPolyDataFELaplaceBeltramiStencil&);  // Not implemented.
  void operator=(const vtkvmtkPolyDataFELaplaceBeltramiStencil&);  // Not implemented.
};

#endif

